# Copyright 2020 by Erick Alexis Alvarez Sanchez, The national meteorological and hydrological service of Peru (SENAMHI).
# All rights reserved.
# This file is part of the pypost package,
# and is released under the "MIT License Agreement". Please see the LICENSE
# file that should have been included as part of this package.

import xarray as xray
import pandas as pd
import numpy as np
from .cpp import cpp
from .cpp_e import cpp_e
import os.path

def datasets_netcdf(do,dm,da,var,f_cal=None,f_apl=None):
    #LEER DATASETS EN FORMATO NETCDF
    cnj_datos=dict()
    for i,j in zip(['do','dm','da'],[do,dm,da]):
        if f_cal==None or f_apl==None:
            #SIN ESPECIFICAR FECHA
            cnj_datos[i]=xray.open_dataset(j)[var]
        else:
            #CON FECHA ESPECIFICADA
            if i=='da':
                cnj_datos[i]=xray.open_dataset(j)[var].sel(time=slice(f_apl[0],f_apl[1]))
            else:
                cnj_datos[i]=xray.open_dataset(j)[var].sel(time=slice(f_cal[0],f_cal[1]))
        #Interpolacion Lineal
        for la,lo in zip(['latitud','latitude','Latitude'],['longitud','longitude','Longitude']):
            if ((la in cnj_datos[i].coords.dims) and (lo in cnj_datos[i].coords.dims)):
                print('cambiando nombre de coordenadas a latlon')
                cnj_datos[i]=cnj_datos[i].rename({la:'lat',lo:'lon'})
            else:
                print('nombres latlon correcto')
        if i!='do':
            if not (np.array_equal(cnj_datos['do']['lat'].values,cnj_datos[i]['lat'].values) and
             np.array_equal(cnj_datos['do']['lon'].values,cnj_datos[i]['lon'].values)):
                print('interpolando latlon')
                cnj_datos[i]=cnj_datos[i].interp(lat=cnj_datos['do']['lat'],lon=cnj_datos['do']['lon'])
            else:
                pass
        #Orden de dimensiones
        cnj_datos[i]=cnj_datos[i].transpose("time", "lat", "lon")
    return cpp(cnj_datos)

def open_station(estaciones,do,dm,da,var,f_cal=None,f_apl=None):
    #LEER DATASETS EN FORMATO NETCDF, OBSERVADOS y COORDENADAS.
    cnj_datos=dict()
    coords=pd.read_csv(estaciones,delimiter=';',index_col=0)
    datos=pd.read_csv(do,delimiter=';',index_col=0)
    datos.index=pd.to_datetime(datos.index,format='%d/%m/%Y')
    datos=datos.loc[f_cal[0]:f_cal[1]]
    if f_cal==None or f_apl==None:
        cnj_datos['dm']=xray.open_dataset(dm)[var]
        cnj_datos['da']=xray.open_dataset(da)[var]
    else:
        cnj_datos['dm']=xray.open_dataset(dm)[var].sel(time=slice(f_cal[0],f_cal[1]))
        cnj_datos['da']=xray.open_dataset(da)[var].sel(time=slice(f_apl[0],f_apl[1]))
    for model_data in ['dm','da']:
        for la,lo in zip(['latitud','latitude','Latitude'],['longitud','longitude','Longitude']):
            if ((la in cnj_datos[model_data].coords.dims) and (lo in cnj_datos[model_data].coords.dims)):
                print('cambiando nombre de coordenadas a latlon')
                cnj_datos[model_data]=cnj_datos[model_data].rename({la:'lat',lo:'lon'})
            else:
                print('nombres latlon correcto')
    #MOMENTANEO-----------------------------------
    #if cnj_datos['dm']['lon'].max()>180:
    #    cnj_datos['dm']['lon']=cnj_datos['dm']['lon']-360.0
    #if cnj_datos['da']['lon'].max()>180:
    #    cnj_datos['da']['lon']=cnj_datos['da']['lon']-360.0
    #--------------------------------------------------------------------------
    cnj_datos['do']=datos
    cnj_datos['coords']=coords
    return cpp_e(cnj_datos)



# Copyright 2020 by Erick Alexis Alvarez Sanchez, The national meteorological and hydrological service of Peru (SENAMHI).
# All rights reserved.
# This file is part of the pypost package,
# and is released under the "MIT License Agreement". Please see the LICENSE
# file that should have been included as part of this package.

import xarray as xray
import numpy as np
import pandas as pd
from .ploteos import correct_mon
from .ploteos_e import correct_e

class cpp_e(object):
    def __init__(self,datos):
        self.data=datos
    def day_scaling(self,acu=False,f_cal=None,f_apl=None):
        #APLICAR SCALING DIARIO---------------------------
        print('realizando Scaling diario')
        cnj_do=pd.DataFrame()
        cnj_dm=pd.DataFrame()
        cnj_da=pd.DataFrame()
        cnj_cor=pd.DataFrame()
        for name in self.data['coords']['nombre']:
            print(name)
            coords=self.data['coords']
            data=self.data['do'][name]
            filtro_mes=data.resample('MS').agg(pd.Series.sum,skipna=False)
            data=data.where(~filtro_mes.reindex(data.index,method='ffill').isna())
            lat=coords.loc[coords['nombre']==name]['lat'].values[0]
            lon=coords.loc[coords['nombre']==name]['lon'].values[0]
            do=data.to_xarray()
            do=do.rename({'index':'time'})
            dm=self.data['dm'].interp(lat=lat,lon=lon)
            dm=dm.drop(['lat','lon'])
            da=self.data['da'].interp(lat=lat,lon=lon)
            da=da.drop(['lat','lon'])
            if f_cal==None or f_apl==None:
                pass
            else:
                do=do.loc[f_cal[0]:f_cal[1]]
                dm=dm.loc[f_cal[0]:f_cal[1]]
                da=da.loc[f_apl[0]:f_apl[1]]
            do['time']=dm['time']
            dm=dm.where(do>-888)
            if acu==True:
                divisor=dm.resample(time='MS').sum('time',skipna=False).groupby('time.month').mean('time')
                divisor=divisor.where(~(divisor<0.01),0.01)
                fc=(do.resample(time='MS').sum('time',skipna=False).groupby('time.month').mean('time')/
                        divisor)
                cor=(da.groupby('time.month')*fc)
            else:
                fc=(do.resample(time='MS').mean('time',skipna=False).groupby('time.month').mean('time')-
                        dm.resample(time='MS').mean('time',skipna=False).groupby('time.month').mean('time'))
                cor=(da.groupby('time.month')+fc)
            cor=cor.drop(['month'])
            cnj_do.loc[:,name]=do.to_dataframe(name=name).iloc[:,0]
            cnj_dm.loc[:,name]=dm.to_dataframe(name=name).iloc[:,0]
            cnj_da.loc[:,name]=da.to_dataframe(name=name).iloc[:,0]
            cnj_cor.loc[:,name]=cor.to_dataframe(name=name).iloc[:,0]
        print('Terminado Scaling diario')
        return correct_e(cnj_cor,cnj_da,cnj_dm,cnj_do,acu=acu)
    def day_eqm(self,acu=False,f_cal=None,f_apl=None):
        self.acumulado=acu
        #APLICAR EQM DIARIO.------------------------------
        print('realizando EQM diario')
        cnj_do=pd.DataFrame()
        cnj_dm=pd.DataFrame()
        cnj_da=pd.DataFrame()
        cnj_cor=pd.DataFrame()
        for name in self.data['coords']['nombre']:
            print(name)
            coords=self.data['coords']
            data=self.data['do'][name]
            filtro_mes=data.resample('MS').agg(pd.Series.sum,skipna=False)
            data=data.where(~filtro_mes.reindex(data.index,method='ffill').isna())
            lat=coords.loc[coords['nombre']==name]['lat'].values[0]
            lon=coords.loc[coords['nombre']==name]['lon'].values[0]
            do=data.to_xarray()
            do=do.rename({'index':'time'})
            dm=self.data['dm'].interp(lat=lat,lon=lon)
            dm=dm.drop(['lat','lon'])
            da=self.data['da'].interp(lat=lat,lon=lon)
            da=da.drop(['lat','lon'])
            if f_cal==None or f_apl==None:
                pass
            else:
                do=do.loc[f_cal[0]:f_cal[1]]
                dm=dm.loc[f_cal[0]:f_cal[1]]
                da=da.loc[f_apl[0]:f_apl[1]]
            do['time']=dm['time']
            dm=dm.where(do>-888)
            quantiles=np.arange(0.01,1,0.01)
            meses=[1,2,3,4,5,6,7,8,9,10,11,12]
            for mon in meses:
                do2=do.loc[do['time.month']==mon]
                dm2=dm.loc[dm['time.month']==mon]
                da2=da.loc[da['time.month']==mon]
                if acu==True:
                    do_f=do2.where(do2>=0.001)
                    dm_f=dm2.where(dm2>=0.001)
                    da_f=da2.where(da2>=0.001)
                else:
                    do_f=do2
                    dm_f=dm2
                    da_f=da2
                datos_his=dm_f
                datos_pro=da_f
                datos_obs_q=do_f.quantile(quantiles,dim='time')
                datos_his_q=dm_f.quantile(quantiles,dim='time')
                datos_pro_q=da_f.quantile(quantiles,dim='time')
                for quan in quantiles:
                    if quan==0.01:
                        datos_his_cor=datos_his.where(datos_his>datos_his_q.sel(quantile=0.02,method='nearest'),
                                                      datos_obs_q.sel(quantile=0.01,method='nearest'))
                    elif quan==0.99:
                        datos_his_cor=datos_his_cor.where(~(datos_his>=datos_his_q.sel(quantile=0.99,method='nearest')),
                                                          datos_obs_q.sel(quantile=0.99,method='nearest'))
                    else:
                        datos_his_cor=datos_his_cor.where(~((datos_his>=datos_his_q.sel(quantile=quan,method='nearest'))&
                                                          (datos_his<datos_his_q.sel(quantile=quan+0.01,method='nearest'))),
                                                          datos_obs_q.sel(quantile=quan,method='nearest'))

                deltas=datos_his_cor.quantile(quantiles,dim='time')-datos_his.quantile(quantiles,dim='time')
                #AÑADIENDO DELTAS DE QUANTILES A LA INFORMACION PROYECTADA.
                for quan in quantiles:
                    if quan==0.01:
                        datos_pro_cor=datos_pro.where(datos_pro>datos_pro_q.sel(quantile=0.02,method='nearest'),
                                                      datos_pro+deltas.sel(quantile=0.01,method='nearest'))
                    elif quan==0.99:
                        datos_pro_cor=datos_pro_cor.where(~(datos_pro>=datos_pro_q.sel(quantile=0.99,method='nearest')),
                                                          datos_pro+deltas.sel(quantile=0.99,method='nearest'))
                    else:
                        datos_pro_cor=datos_pro_cor.where(~((datos_pro>=datos_pro_q.sel(quantile=quan,method='nearest'))&
                                                          (datos_pro<datos_pro_q.sel(quantile=quan+0.01,method='nearest'))),
                                                          datos_pro+deltas.sel(quantile=quan,method='nearest'))
                if mon==1:
                    datos_his_cor2=datos_his_cor
                    datos_pro_cor2=datos_pro_cor
                else:
                    datos_his_cor2=xray.concat([datos_his_cor2,datos_his_cor],dim='time')
                    datos_pro_cor2=xray.concat([datos_pro_cor2,datos_pro_cor],dim='time')
            datos_his_cor2=datos_pro_cor2.sortby('time',ascending=True)
            datos_pro_cor2=datos_pro_cor2.sortby('time',ascending=True)
            if acu==True:
                #cor=da.where(~(da<0.1),0)
                #cor=cor.where(~(cor>=0.1),datos_pro_cor2)
                cor=datos_pro_cor2
                cor=cor.where(cor.notnull(),da)
                cor=cor.where(~(cor<0),0)
            else:
                cor=datos_pro_cor2
            cnj_do.loc[:,name]=do.to_dataframe(name=name).iloc[:,0]
            cnj_dm.loc[:,name]=dm.to_dataframe(name=name).iloc[:,0]
            cnj_da.loc[:,name]=da.to_dataframe(name=name).iloc[:,0]
            cnj_cor.loc[:,name]=cor.to_dataframe(name=name).iloc[:,0]
        #NOTA: LAS ADVERTENCIAS SALEN DEBIDO A QUE NO HAY DATOS EN OCEANO (POR PARTE DEL OBSERVADO)
        print('realizando EQM diario')
        return correct_e(cnj_cor,cnj_da,cnj_dm,cnj_do,acu=acu)



# Copyright 2020 by Erick Alexis Alvarez Sanchez, The national meteorological and hydrological service of Peru (SENAMHI).
# All rights reserved.
# This file is part of the pypost package,
# and is released under the "MIT License Agreement". Please see the LICENSE
# file that should have been included as part of this package.

import seaborn as sns
from sklearn.linear_model import LinearRegression
import numpy as np
import matplotlib.pyplot as plt
class correct_e:
    def __init__(self,cor,da,dm,do,acu=False):
        self.observados=do
        self.historicos=dm
        self.proyectados=da
        self.corregidos=cor



# Copyright 2020 by Erick Alexis Alvarez Sanchez, The national meteorological and hydrological service of Peru (SENAMHI).
# All rights reserved.
# This file is part of the pypost package,
# and is released under the "MIT License Agreement". Please see the LICENSE
# file that should have been included as part of this package.
import xarray as xray
import numpy as np
from .ploteos import correct_mon

class cpp(object):
    def __init__(self,datos):
        self.data=datos
    def month_scaling(self,acu=False,f_cal=None,f_apl=None):
        #APLICAR SCALING MENSUAL---------------------------------
        print('realizando Scaling mensual')
        if f_cal==None or f_apl==None:
            pass
        else:
            self.data['do']=self.data['do'].loc[f_cal[0]:f_cal[1]]
            self.data['dm']=self.data['dm'].loc[f_cal[0]:f_cal[1]]
            self.data['da']=self.data['da'].loc[f_apl[0]:f_apl[1]]
        if acu==True:
            divisor=self.data['dm'].groupby('time.month').mean('time')
            divisor=divisor.where(~(divisor<0.01),0.01)
            fc=(self.data['do'].groupby('time.month').mean('time')/
                    divisor)
            cor=(self.data['da'].groupby('time.month')*fc)
        else:
            fc=(self.data['do'].groupby('time.month').mean('time')-
                    self.data['dm'].groupby('time.month').mean('time'))
            cor=(self.data['da'].groupby('time.month')+fc)
        print('Terminado Scaling mensual')
        return correct_mon(cor,self.data['da'],self.data['do'],self.data['dm'],acu=acu)
    def day_scaling(self,acu=False,f_cal=None,f_apl=None):
        #APLICAR SCALING DIARIO---------------------------
        self.acumulado=acu
        print('realizando Scaling diario')
        if f_cal==None or f_apl==None:
            pass
        else:
            self.data['do']=self.data['do'].loc[f_cal[0]:f_cal[1]]
            self.data['dm']=self.data['dm'].loc[f_cal[0]:f_cal[1]]
            self.data['da']=self.data['da'].loc[f_apl[0]:f_apl[1]]
        if acu==True:
            divisor=self.data['dm'].resample(time='MS').sum('time',skipna=False).groupby('time.month').mean('time')
            divisor=divisor.where(~(divisor<0.01),0.01)
            fc=(self.data['do'].resample(time='MS').sum('time',skipna=False).groupby('time.month').mean('time')/
                    divisor)
            cor=(self.data['da'].groupby('time.month')*fc)
        else:
            fc=(self.data['do'].resample(time='MS').mean('time',skipna=False).groupby('time.month').mean('time')-
                    self.data['dm'].resample(time='MS').mean('time',skipna=False).groupby('time.month').mean('time'))
            cor=(self.data['da'].groupby('time.month')+fc)
        print('Terminado Scaling diario')
        return correct_mon(cor,self.data['da'],self.data['do'],self.data['dm'],acu=acu)
    def day_eqm(self,acu=False,f_cal=None,f_apl=None):
        self.acumulado=acu
        #APLICAR EQM DIARIO.------------------------------
        print('realizando EQM diario')
        if f_cal==None or f_apl==None:
            pass
        else:
            self.data['do']=self.data['do'].loc[f_cal[0]:f_cal[1]]
            self.data['dm']=self.data['dm'].loc[f_cal[0]:f_cal[1]]
            self.data['da']=self.data['da'].loc[f_apl[0]:f_apl[1]]
        quantiles=np.arange(0.01,1,0.01)
        meses=[1,2,3,4,5,6,7,8,9,10,11,12]
        if acu==True:
            print("acotando numeros cercanos a cero")
            do_mayor=self.data['do'].where(self.data['do']>=0.001)
            dm_mayor=self.data['dm'].where(self.data['dm']>=0.001)
            da_mayor=self.data['da'].where(self.data['da']>=0.001)
            print("terminando acotacion")
        for mon in meses:
            if acu==True:
                do_f=do_mayor.loc[do_mayor['time.month']==mon]
                dm_f=dm_mayor.loc[dm_mayor['time.month']==mon]
                da_f=da_mayor.loc[da_mayor['time.month']==mon]
                #do_f=do2.where(do2>=0.1)
                #dm_f=dm2.where(dm2>=0.1)
                #da_f=da2.where(da2>=0.1)
            else:
                do_f=self.data['do'].loc[self.data['do']['time.month']==mon]
                dm_f=self.data['dm'].loc[self.data['dm']['time.month']==mon]
                da_f=self.data['da'].loc[self.data['da']['time.month']==mon]
            datos_his=dm_f
            datos_pro=da_f
            datos_obs_q=do_f.quantile(quantiles,dim='time').astype('float32')
            datos_his_q=dm_f.quantile(quantiles,dim='time').astype('float32')
            datos_pro_q=da_f.quantile(quantiles,dim='time').astype('float32')
            for quan in quantiles:
                if quan==0.01:
                    datos_his_cor=datos_his.where(datos_his>datos_his_q.sel(quantile=0.02,method='nearest'),
                                                  datos_obs_q.sel(quantile=0.01,method='nearest'))
                elif quan==0.99:
                    datos_his_cor=datos_his_cor.where(~(datos_his>=datos_his_q.sel(quantile=0.99,method='nearest')),
                                                      datos_obs_q.sel(quantile=0.99,method='nearest'))
                else:
                    datos_his_cor=datos_his_cor.where(~((datos_his>=datos_his_q.sel(quantile=quan,method='nearest'))&
                                                      (datos_his<datos_his_q.sel(quantile=quan+0.01,method='nearest'))),
                                                      datos_obs_q.sel(quantile=quan,method='nearest'))

            deltas=datos_his_cor.quantile(quantiles,dim='time').astype('float32')-datos_his.quantile(quantiles,dim='time').astype('float32')

            #AÑADIENDO DELTAS DE QUANTILES A LA INFORMACION PROYECTADA.
            for quan in quantiles:
                if quan==0.01:
                    datos_pro_cor=datos_pro.where(datos_pro>datos_pro_q.sel(quantile=0.02,method='nearest'),
                                                  datos_pro+deltas.sel(quantile=0.01,method='nearest'))
                elif quan==0.99:
                    datos_pro_cor=datos_pro_cor.where(~(datos_pro>=datos_pro_q.sel(quantile=0.99,method='nearest')),
                                                      datos_pro+deltas.sel(quantile=0.99,method='nearest'))
                else:
                    datos_pro_cor=datos_pro_cor.where(~((datos_pro>=datos_pro_q.sel(quantile=quan,method='nearest'))&
                                                      (datos_pro<datos_pro_q.sel(quantile=quan+0.01,method='nearest'))),
                                                      datos_pro+deltas.sel(quantile=quan,method='nearest'))
            if mon==1:
                datos_his_cor2=datos_his_cor
                datos_pro_cor2=datos_pro_cor
            else:
                datos_his_cor2=xray.concat([datos_his_cor2,datos_his_cor],dim='time')
                datos_pro_cor2=xray.concat([datos_pro_cor2,datos_pro_cor],dim='time')
        datos_his_cor2=datos_pro_cor2.sortby('time',ascending=True)
        datos_pro_cor2=datos_pro_cor2.sortby('time',ascending=True)
        if acu==True:
            #cor=self.data['da'].where(~(self.data['da']<0.001),0)
            cor=datos_pro_cor2
            cor=cor.where(cor.notnull(),self.data['da'])
            #cor=cor.where(~(cor>=0.001),datos_pro_cor2)
            cor=cor.where(~(cor<0),0)
            cor=cor.where(self.data['do'].isel(time=0)>-888) #Para tener solo los datos en lugares donde habia observados
            #cor=cor.where(~(cor<0.001),0)
        else:
            cor=datos_pro_cor2
            cor=cor.where(self.data['do'].isel(time=0)>-888) #Para tener solo los datos en lugares donde habia observados
        #NOTA: LAS ADVERTENCIAS SALEN DEBIDO A QUE NO HAY DATOS EN OCEANO (POR PARTE DEL OBSERVADO)
        print('Terminado EQM diario')
        return correct_mon(cor,self.data['da'],self.data['do'],self.data['dm'],acu=acu)



# Copyright 2020 by Erick Alexis Alvarez Sanchez, The national meteorological and hydrological service of Peru (SENAMHI).
# All rights reserved.
# This file is part of the pypost package,
# and is released under the "MIT License Agreement". Please see the LICENSE
# file that should have been included as part of this package.

from setuptools import setup

setup(
    name='pypost',
    version='1.0.0.2',
    description='Primera version de pypost!',
    packages = ['pypost'],
    license='MIT',
    author_email='erick.alvarez.met@gmail.com',
    download_url='https://github.com/3r1ck10/pypost/archive/v1.0.0.2.tar.gz',
    install_requires=['numpy',
    'pandas','matplotlib','xarray','seaborn','scikit-learn','netcdf4'],
    classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
  ],
)



# Copyright 2020 by Erick Alexis Alvarez Sanchez, The national meteorological and hydrological service of Peru (SENAMHI).
# All rights reserved.
# This file is part of the pypost package,
# and is released under the "MIT License Agreement". Please see the LICENSE
# file that should have been included as part of this package.

import seaborn as sns
from sklearn.linear_model import LinearRegression
import numpy as np
import matplotlib.pyplot as plt
class correct_mon:
    def __init__(self,cor,da,do,dm,acu=False):
        self.corregido=cor
        self.no_corregido=da
        self.observado=do
        self.historico=dm
        self.acu=acu
    def plot(self):
        if self.acu==True:
            cmap='viridis'
        else:
            cmap='jet'
        fig=plt.figure(figsize=(10,10))
        gs=fig.add_gridspec(4,4,hspace=0.3)
        corregido=self.corregido
        #AGREGAR -800 PARA Q LA MASCARA SEA IGUAL AL CORREGIDO
        no_corregido=self.no_corregido.where(corregido>-800)
        #CAMBIAR PROMEDIOS A SUMAS EN CASO DE PRECIPITACION
        if self.acu==True:
            dc=corregido.resample(time='Q-FEB').sum('time',skipna=False).groupby('time.season').mean('time')
            dm=no_corregido.resample(time='Q-FEB').sum('time',skipna=False).groupby('time.season').mean('time')
            vmin=0
            vmax=dc.max().values
        else:
            dc=corregido.groupby('time.season').mean('time')
            dm=no_corregido.groupby('time.season').mean('time')
            vmin=dc.min().values
            vmax=dc.max().values
        cols=range(4)
        rows=range(2)
        seasons=['DJF','MAM','JJA','SON']
        #PLOTEANDO TRIMESTRES
        for row,dat,datname in zip(rows,[dc,dm],['corregido','no-corregido']):
            for col,season in zip(cols,seasons):
                ax=fig.add_subplot(gs[row,col])
                dato=dat.sel(season=season).plot(ax=ax,vmin=vmin,vmax=vmax,cmap=cmap,add_colorbar=False)
                if datname=='corregido':
                    ax.set_title(season)
                else:
                    ax.set_title('')
                if season=='DJF':
                    ax.set_ylabel(datname)
                else:
                    ax.set_ylabel('')
                ax.set_xlabel('')
        cbar_ax1 = fig.add_axes([0.93, 0.52, 0.02, 0.36])
        cbar1=fig.colorbar(dato,cax=cbar_ax1)
        #--------------------------------------------------------------------
        #PLOTEANDO LA SERIE ANUAL
        if self.acu==True:
            dc=corregido.groupby('time.year').sum('time',skipna=False).mean(dim=['lat','lon'])
            dm=no_corregido.groupby('time.year').sum('time',skipna=False).mean(dim=['lat','lon'])
        else:
            dc=corregido.groupby('time.year').mean('time').mean(dim=['lat','lon'])
            dm=no_corregido.groupby('time.year').mean('time').mean(dim=['lat','lon'])
        ax=fig.add_subplot(gs[2,:2])
        dc.plot(ax=ax)
        dm.plot(ax=ax)
        ax.set_title('Serie anual')
        ax.set_xlabel('')
        #------------------------------------------------------------------------
        #PLOTEANDO EL CICLO ANUAL
        if self.acu==True:
            dc=corregido.resample(time='MS').sum('time',skipna=False).groupby('time.month').mean('time').mean(dim=['lat','lon'])
            dm=no_corregido.resample(time='MS').sum('time',skipna=False).groupby('time.month').mean('time').mean(dim=['lat','lon'])
        else:
            dc=corregido.groupby('time.month').mean('time').mean(dim=['lat','lon'])
            dm=no_corregido.groupby('time.month').mean('time').mean(dim=['lat','lon'])
        ax=fig.add_subplot(gs[3,:2])
        dc.plot(ax=ax)
        dm.plot(ax=ax)
        ax.set_title('Ciclo anual')
        plt.legend(['corr','no-corr'])
        #-----------------------------------------------------------------------
        #PLOTEANDO IMAGENES ANUALES
        if self.acu==True:
            dc=corregido.groupby('time.year').sum('time',skipna=False).mean('year')
            dm=no_corregido.groupby('time.year').sum('time',skipna=False).mean('year')
            vmin=0
            vmax=dc.max().values
        else:
            dc=corregido.groupby('time.year').mean('time').mean('year')
            dm=no_corregido.groupby('time.year').mean('time').mean('year')
            vmin=dc.min().values
            vmax=dc.max().values
        for col,dat,datname in zip([2,3],[dc,dm],['corregido_anual','no-corregido_anual']):
            ax=fig.add_subplot(gs[2,col])
            dato=dat.plot(ax=ax,vmin=vmin,vmax=vmax,cmap=cmap,add_colorbar=False)
            ax.set_title(datname)
            ax.set_ylabel('')
            ax.set_xlabel('')
        cbar_ax2 = fig.add_axes([0.93, 0.32, 0.02, 0.16])
        cbar2=fig.colorbar(dato,cax=cbar_ax2)
        #-------------------------------------------------------------------
        #PLOTEANDO DISTRIBUCION ANUAL
        ax=fig.add_subplot(gs[3,2])
        sns.distplot(np.ndarray.flatten(dc.values),kde=False,bins=15,ax=ax)
        sns.distplot(np.ndarray.flatten(dm.values),kde=False,bins=15,ax=ax)
        ax.set_title('Distribucion espacial anual')
        #---------------------------------TABLA----------------------------------
        #PLOTEANDO TABLA
        if self.acu==True:
            y_cor=corregido.groupby('time.year').sum('time',skipna=False).mean(dim=['lat','lon']).values.reshape(-1, 1)
            y_mod=no_corregido.groupby('time.year').sum('time',skipna=False).mean(dim=['lat','lon']).values.reshape(-1, 1)
        else:
            y_cor=corregido.groupby('time.year').mean('time').mean(dim=['lat','lon']).values.reshape(-1, 1)
            y_mod=no_corregido.groupby('time.year').mean('time').mean(dim=['lat','lon']).values.reshape(-1, 1)
        x=np.arange(1,len(y_cor)+1).reshape(-1,1)
        reg_cor=LinearRegression().fit(x,y_cor)
        reg_mod=LinearRegression().fit(x,y_mod)
        #_-----------------------
        ax=fig.add_subplot(gs[3,3])
        col_labels = ['Cor','no_Cor']
        row_labels = ['Tenden.','Prom.','Max','Min','SD']
        table_vals = [[reg_cor.coef_[0][0],reg_mod.coef_[0][0]],
                      [dc.mean().values,dm.mean().values],
                      [dc.max().values,dm.max().values],
                      [dc.min().values,dm.min().values],
                      [dc.std().values,dm.std().values]]
        table_vals2 = [['%.2f' % j for j in i] for i in table_vals]


        the_table = ax.table(cellText=table_vals2,
                              colWidths=[0.05] * 3,
                              rowLabels=row_labels,
                              colLabels=col_labels,
                              loc='center right',cellLoc='center')
        the_table.auto_set_font_size(False)
        the_table.set_fontsize(10)
        the_table.scale(7, 1.6)
        ax.tick_params(axis='x', which='both', bottom=False, top=False, labelbottom=False)
        ax.tick_params(axis='y', which='both', right=False, left=False, labelleft=False)
        ax.set_title('resumen anual')
        for pos in ['right','top','bottom','left']:
            ax.spines[pos].set_visible(False)
        plt.close(fig)
        return fig
